@@ -27,7 +27,7 @@ use constant DATASET_HASH_LIMIT => 5001;
 		# region, and a hash returned with potentially a score for each basepair. 
 		# This may become unwieldy for very large regions, which may be better 
 		# served by separate database queries for each window.
-my $VERSION = 1.24;
+my $VERSION = 1.25;
 
 print "\n This script will collect binned values across features\n\n";
 
@@ -141,6 +141,11 @@ if ($infile) {
 	else {
 		$main_database = $Data->database;
 	}
+	
+	# update feature type as necessary
+	if (not defined $Data->feature and not defined $Data->type_column and defined $feature) {
+		$Data->feature($feature);
+	}
 }
 else {
 	# generate a new file
@@ -390,6 +395,7 @@ sub parallel_execution {
 		
 		# re-open database objects to make them clone safe
 		# pass second true to avoid cached database objects
+		my $db = $Data->open_database(1);
 		if ($data_database) {
 			$ddb = open_db_connection($data_database, 1);
 		}
